import pokebattle
import asyncio

OMEGALUL = "<:omega:844632595110166528>"


class _Player:
    def __init__(self, username):
        self.username = username
        self.poke_names = []
        self.cur_index = 0
        self.cur_hp = 0

    def declare_pokes(self, poke_names):
        self.poke_names = list(poke_names)

    def get_cur_poke(self):
        return self.poke_names[self.cur_index]

    def has_more_pokes(self):
        return self.cur_index < len(self.poke_names)

    def get_cur_hp(self):
        return max(0, self.cur_hp)

    def hasDeclared(self):
        return len(self.poke_names) > 0

    def next_poke(self):
        self.cur_index += 1


class Battle:
    def __init__(self, starter_name, time, decl_max):
        self.time = time
        self.is_happening = False
        self.players = [_Player(starter_name), None]
        self.players_info = []
        self.decl_max = decl_max

    def acceptPlayerTwo(self, username):
        self.players[1] = _Player(username)

    def getP1Name(self):
        return self.players[0].username

    def getP2Name(self):
        return self.players[1] and self.players[1].username

    def getPlayerByName(self, name):
        if name == self.getP1Name():
            return self.players[0]
        elif name == self.getP2Name():
            return self.players[1]
        return False

    def _getPlayerByIndex(self, index):
        return self.players[index]

    def _getKickoffCopy(self):
        p0, p1 = self.players
        p0poke, p1poke = p0.get_cur_poke(), p1.get_cur_poke()
        return (
            f"{p0.username}'s **{p0poke} ({p0.get_cur_hp()} hp)** is now battling "
            f"{p1.username}'s **{p1poke} ({p1.get_cur_hp()} hp)**!"
        )

    def _getHpInfoCopy(self):
        FULL_HP = "■"
        MISSING_HP = "□"
        p0, p1 = self.players
        p0poke, p1poke = p0.get_cur_poke(), p1.get_cur_poke()
        p0_missing_hp = pokebattle.get_hp(p0poke) - p0.get_cur_hp()
        p1_missing_hp = pokebattle.get_hp(p1poke) - p1.get_cur_hp()
        p0_missing_hp_count = int(p0_missing_hp / (pokebattle.get_hp(p0poke) / 8))
        p1_missing_hp_count = int(p1_missing_hp / (pokebattle.get_hp(p1poke) / 8))
        p0bar = FULL_HP * (8 - p0_missing_hp_count) + MISSING_HP * p0_missing_hp_count
        p1bar = FULL_HP * (8 - p1_missing_hp_count) + MISSING_HP * p1_missing_hp_count
        p0_hp_text = f"{p0.get_cur_hp()}/{pokebattle.get_hp(p0poke)}"
        p1_hp_text = f"{p1.get_cur_hp()}/{pokebattle.get_hp(p1poke)}"

        return f"** **{p0bar}      {p1bar}\n** **{p0_hp_text:<18}{p1_hp_text}"

    """
    battle_script_shape: {
        'kickoff': {
            'msg': <string>,
            'img_list': <list(string)>
        },
        'actions': <list(tuple(description, hp_info))>,
        'outcome': <string>,
    }
    """

    def _start_script(self):
        p0, p1 = self.players
        p0name = p0.username
        p1name = p1.username
        p0pokes, p1pokes = p0.poke_names, p1.poke_names
        return [
            f"**{p0name}'s pokemon:**",
            p0pokes,
            f"**{p1name}'s pokemon:**",
            p1pokes,
        ]

    def _turn_script(self, attacker_index):
        p0, p1 = self.players
        p0poke, p1poke = p0.get_cur_poke(), p1.get_cur_poke()
        if attacker_index == 0:
            atk_name = p0poke
            def_name = p1poke
        else:
            atk_name = p1poke
            def_name = p0poke
        actions = []

        (
            total_dmg,
            did_it_miss,
            did_it_crit,
            move_typing,
            effectiveness,
        ) = pokebattle.deal_damage(atk_name, def_name)
        # tuple containing (total damage, crit or not, damage type, and effectiveness)
        if effectiveness > 1:
            effectiveness_str = "It's **super effective**!"
        elif effectiveness < 1:
            effectiveness_str = "It's **not very effective**..."
        else:
            effectiveness_str = ""

        self._getPlayerByIndex(1 - attacker_index).cur_hp -= total_dmg

        action_copy = f"**{atk_name}**"
        if total_dmg == 0:
            action_copy += "'s attack missed!"
        else:
            if not did_it_crit:
                action_copy += f" deals **{total_dmg}** damage to **{def_name}**"
            else:
                action_copy += (
                    f" crits **{def_name}** for a total of **{total_dmg}** damage"
                )
            action_copy += f" with a **{move_typing}**-type move! {effectiveness_str}"

        actions.append((action_copy, self._getHpInfoCopy()))
        return actions

    def _round_script(self):
        p0, p1 = self.players
        p0poke, p1poke = p0.get_cur_poke(), p1.get_cur_poke()
        round_info = {
            "kickoff": {},
            "actions": [],
        }
        attacker_index = pokebattle.goes_first(p0poke, p1poke)
        defender_index = 1 - attacker_index

        # kick off message
        round_info["kickoff"]["msg"] = self._getKickoffCopy()
        round_info["kickoff"]["img_list"] = [p0poke, p1poke]
        hasFainted = False
        while not hasFainted:
            round_info["actions"].extend(self._turn_script(attacker_index))
            def_player = self._getPlayerByIndex(defender_index)
            if def_player.get_cur_hp() <= 0:
                round_info["outcome"] = f"**{def_player.get_cur_poke()}** has fainted!"
                hasFainted = True
            else:
                attacker_index = 1 - attacker_index
                defender_index = 1 - defender_index

        return (
            self._getPlayerByIndex(attacker_index),
            self._getPlayerByIndex(defender_index),
            round_info,
        )

    def _compute_battle_script(self):
        battle_log = []
        p0, p1 = self._getPlayerByIndex(0), self._getPlayerByIndex(1)
        p0.cur_hp, p1.cur_hp = pokebattle.get_hp(p0.get_cur_poke()), pokebattle.get_hp(
            p1.get_cur_poke()
        )

        while all([p.has_more_pokes() for p in self.players]):
            winner, loser, round_info = self._round_script()
            loser.next_poke()
            if loser.has_more_pokes():
                loser.cur_hp = pokebattle.get_hp(loser.get_cur_poke())
            battle_log.append(round_info)
        # handle battle end messages
        battle_log.append(
            (
                (
                    f"**{loser.username}** got R{OMEGALUL}LLED by **{winner.username}**"
                    f"! **{loser.username}** scurried to a POKEMON CENTER, "
                    "protecting their exhausted Pokemon from further harm..."
                ),
                winner.poke_names,
            )
        )
        return battle_log

    async def compute_and_send_script(self, sendFn):
        p0start, p0pokes, p1start, p1pokes = self._start_script()
        await sendFn(p0start, p0pokes)
        await sendFn(p1start, p1pokes)
        await asyncio.sleep(5)
        await sendFn("**Let the games begin!**")
        battle_script = self._compute_battle_script()
        for round_info in battle_script[:-1]:
            await sendFn(
                round_info["kickoff"]["msg"], round_info["kickoff"]["img_list"]
            )
            last_msg = None
            for description, hp_info in round_info["actions"]:
                if not self.is_happening:
                    await sendFn("The battle has been aborted. Pepehands")
                    return False
                if not last_msg:
                    last_msg = await sendFn(f"{hp_info}\n{description}\n** **", [])
                else:
                    prev_hp_info = "\n".join(last_msg.content.splitlines()[:-2])
                    await last_msg.edit(content=f"{prev_hp_info}\n** **\n** **")
                    await asyncio.sleep(1)
                    await last_msg.edit(content=f"{hp_info}\n{description}\n** **")
                await asyncio.sleep(4)
            await last_msg.delete()
            await sendFn(round_info["outcome"])
        final_msg, final_img_list = battle_script[-1]
        await sendFn(final_msg, final_img_list)
        return True
