import discord
import backend

client = discord.Client()
roles = {}

allowed_dm_commands = ["!coin", "!inv", "!whohas", "!flex", "!declare"]


def isPermitted(is_dm, cmd, user):
    if cmd not in backend.getCommands():
        return False, "Invalid command."
    if is_dm:
        return (
            cmd in allowed_dm_commands,
            f"Cannot use {cmd} in a DM, go to #nickcoin instead.",
        )
    if not backend.user_exists(user) and cmd != "!start":
        return False, "You don't have an account, !start to get started."
    if cmd == "!give":
        return user.top_role == roles["Owner"], "You can't do that"
    return user.top_role >= roles["Friend"], "You can't do that"


@client.event
# called when the bot has connected to discord
async def on_ready():
    global roles
    if not client.guilds:
        raise Exception("bot isnt in any servers")
    server = client.guilds[0]
    for role in server.roles:
        roles[role.name] = role
    print("Logged in as {0.user}".format(client))


@client.event
# called when any message is sent
async def on_message(message):
    is_dm = message.channel.type.name == "private"
    user = message.author

    if user == client.user:  # client.user is the bot
        return
    if not roles:
        return

    if not is_dm and message.channel.name not in ["nickcoin", "testing"]:
        return

    async def send(s, *args, file=None, delete_after=None):
        args = ["**" + str(a) + "**" if a else "" for a in args]
        sent_message = await message.channel.send(
            s.format(*args), file=file, delete_after=delete_after
        )
        return sent_message

    contents = message.content.split(" ")
    command = contents[0]
    if not command or command[0] != "!":
        return
    permitted, errortxt = isPermitted(is_dm, command, user)
    if not permitted:
        await send(errortxt)
        return
    # user is in db, has correct role, and the command is valid
    commandFn = backend.getCommands()[command]
    needsPersist = await commandFn(send, contents, user)
    if needsPersist:
        backend.persist()


backend.load_db()
client.run("ODQxMTg3MjY4NDEyNzAyNzYx.YJjGsQ._yn321sfpUAD9GJ7ndJcSlcHGZ4")
