import requests
import json

SELECTED_GEN = "4"

gen_poke_ids = set()


def write_image(poke_id, sprite_url):
    img_data = requests.get(sprite_url).content
    with open(f"sprites/{poke_id}.png", "wb") as handler:
        handler.write(img_data)


for gen in [1, 2, 3, 4]:
    print(f"starting gen {gen}")
    r = requests.get(f"https://pokeapi.co/api/v2/generation/{gen}/")
    gen_pokemon = r.json()["pokemon_species"]

    for poke in gen_pokemon:
        r = requests.get(poke["url"])
        data = r.json()
        gen_poke_ids.add(data["id"])

    print(f"finished gen {gen}. Total pokemon ids: {len(gen_poke_ids)}")

final_dict = dict()
name2id = dict()
poke_evo = list()

with open("../data/pokemon_evolutions.json", "r") as fp:
    poke_evo = json.load(fp)[SELECTED_GEN]
"""
for gen in ["1", "2", "3", "4"]:
    selected = poke_evos[gen]
    new_selected = []
    for evo in selected:
        if isinstance(evo, list):
            new_selected.append(evo[0])
        else:
            new_selected.append(evo)
    poke_evos[gen] = new_selected
"""


def search_chain(chain, poke_name):
    while True:
        if len(chain["evolutions"]) == 0:
            break
        elif chain["pokemon"] == poke_name:
            return chain
        else:
            possible_ans = None
            for next_evo in chain["evolutions"]:
                next_evo_possible_base = search_chain(next_evo, poke_name)
                if next_evo_possible_base is not None:
                    possible_ans = next_evo_possible_base
            return possible_ans
    return None


def convert_base_to_list(chain_base):
    ans = []
    while True:
        ans.append(chain_base["pokemon"])
        if len(chain_base["evolutions"]) == 0:
            break
    return ans


def find_evolution_chain(poke_evos, poke_name):
    for chain in poke_evos:
        maybe_base = search_chain(chain, poke_name)
        if maybe_base is not None:
            return [pokemon["pokemon"] for pokemon in maybe_base["evolutions"]]
    return ["-1"]


name2id["-1"] = -1

for pokemon_id in gen_poke_ids:
    r = requests.get(f"https://pokeapi.co/api/v2/pokemon/{pokemon_id}/")
    data = r.json()
    poke_name_to_use = data["name"]
    if poke_name_to_use == "wormadam-plant":
        poke_name_to_use = "wormadam"
    poke_evolution_chain = find_evolution_chain(poke_evo, poke_name_to_use)
    pokedata = dict()
    pokedata["name"] = poke_name_to_use
    pokedata["sprite"] = f'sprites/{data["id"]}.png'
    pokedata["rarity"] = 0
    pokedata["stats"] = {
        poke_stat["stat"]["name"]: poke_stat["base_stat"] for poke_stat in data["stats"]
    }
    pokedata["types"] = [poke_type["type"]["name"] for poke_type in data["types"]]
    pokedata["evolution"] = poke_evolution_chain
    final_dict[data["id"]] = pokedata
    name2id[poke_name_to_use] = data["id"]
    # write_image(data['id'], data['sprites']['front_default'])

    spokedata = dict()
    spokedata["name"] = "shiny " + poke_name_to_use
    spokedata["sprite"] = f'sprites/{data["id"] + 1000}.png'
    spokedata["rarity"] = 0
    spokedata["stats"] = {
        poke_stat["stat"]["name"]: poke_stat["base_stat"] for poke_stat in data["stats"]
    }
    spokedata["types"] = [poke_type["type"]["name"] for poke_type in data["types"]]
    spokedata["evolution"] = poke_evolution_chain
    final_dict[data["id"] + 1000] = spokedata
    name2id["shiny " + poke_name_to_use] = data["id"] + 1000
    # write_image(data['id'] + 1000, data['sprites']['front_shiny']);

final_dict = {
    poke_id: {
        **poke_value,
        "evolution": [
            name2id[pokemon_name] for pokemon_name in poke_value["evolution"]
        ],
    }
    for poke_id, poke_value in final_dict.items()
}

with open("../data/pokedex.json", "w") as fp:
    json.dump({int(x): final_dict[x] for x in final_dict.keys()}, fp, sort_keys=True)
