import math
import random
import json
import pokeapi

typing_chart = []
with open("data/typing_chart.json", "r") as file:
    data = file.read()
    if data:
        typing_chart = json.loads(data)
    else:
        raise Exception("no data in pokedex.json")


def type_modifier(atk_type, def_type):
    damage_info = [
        dmg_info for dmg_info in typing_chart if dmg_info["name"] == atk_type
    ][0]
    modifier = 1.0
    if def_type in damage_info["immunes"]:
        modifier = 0.25
    elif def_type in damage_info["weaknesses"]:
        modifier = 0.5
    elif def_type in damage_info["strengths"]:
        modifier = 2.0
    return modifier


def get_hp(poke_name):
    return pokeapi.getBattlePoke(pokeapi.nameToID[poke_name])["stats"]["hp"]


def goes_first(p0_poke_name, p1_poke_name):
    p0_poke_stats = pokeapi.getStats(pokeapi.nameToID[p0_poke_name])
    p1_poke_stats = pokeapi.getStats(pokeapi.nameToID[p1_poke_name])
    if p0_poke_stats["speed"] > p1_poke_stats["speed"]:
        return 0
    elif p0_poke_stats["speed"] < p1_poke_stats["speed"]:
        return 1
    return 0 if random.random() >= 0.5 else 1


def deal_damage(attacking_poke_name, defending_poke_name):
    def normalized_power(rarity):
        power = math.log10(rarity) * 30 + 40
        power = min(power, 135)
        rounded_power = 5 * round(power / 5)
        return rounded_power

    def miss_chance(rarity):
        miss_chance = math.log10(min(rarity, 135) + 100) * 30 + 40
        miss_chance /= 100
        return miss_chance - 1

    atk_poke = pokeapi.getBattlePoke(pokeapi.nameToID[attacking_poke_name])
    def_poke = pokeapi.getBattlePoke(pokeapi.nameToID[defending_poke_name])

    # fn(rarity_score) x A/D x typing score
    total = atk_poke["stats"]["attack"] + atk_poke["stats"]["special-attack"]
    target = random.randint(1, total)
    damage_class = (
        "attack" if target < atk_poke["stats"]["attack"] else "special-attack"
    )
    damage_amount = atk_poke["stats"][damage_class]
    num_types = len(atk_poke["types"])
    damage_type = atk_poke["types"][random.randint(0, num_types) % num_types]

    defending_class = damage_class.replace("attack", "defense")
    defending_amount = def_poke["stats"][defending_class]

    damage_mods = [
        type_modifier(damage_type, def_type) for def_type in def_poke["types"]
    ]
    type_mod = 1
    for mod in damage_mods:
        type_mod *= mod

    crit_mod = 1.0
    did_crit = False
    if (
        random.randint(0, 255)
        < pokeapi.getStats(pokeapi.nameToID[attacking_poke_name])["speed"] / 4
    ):
        crit_mod = 2.0
        did_crit = True

    effective_damage = round(
        normalized_power(atk_poke["rarity"])
        * damage_amount
        / defending_amount
        * type_mod
        * crit_mod
    )

    did_miss = False
    if random.random() < miss_chance(atk_poke["rarity"]):
        did_miss = True
        effective_damage = 0

    return effective_damage, did_miss, did_crit, damage_type, type_mod
