import numpy as np
import math
from nickcoin import pokeapi
import unittest


class TestRolling(unittest.TestCase):
    # runs chi-squared-like test
    # ensure probabilies of rolling are proportional to 1 / rarity
    def test_roll(self):
        num_trials = 100000
        ids, probs = pokeapi._idsToProb[1]
        outcomes = {}
        for id in ids:
            outcomes[id] = 0
        for _ in range(num_trials):
            outcomes[int(pokeapi.roll(1)) % 1000] += 1
        ratios = []
        for id, weight in zip(ids, probs):
            ratios.append(outcomes[id] / weight)
        self.assertAlmostEqual(
            np.std(ratios) / math.sqrt(num_trials),
            0,
            None,
            "Rolling stddev was too high",
            5,
        )
        self.assertLess(outcomes[int(pokeapi.getID("mew"))], 10)


if __name__ == "__main__":
    unittest.main()
